'use strict';

const path = require('node:path');
const fs = require('node:fs');
const createCac = require('cac');
const index = require('./index.cjs');
require('node:fs/promises');
require('postcss');
require('postcss-import');
require('@babel/types');
require('@babel/generator');
require('node:url');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const path__default = /*#__PURE__*/_interopDefaultCompat(path);
const fs__default = /*#__PURE__*/_interopDefaultCompat(fs);
const createCac__default = /*#__PURE__*/_interopDefaultCompat(createCac);

const cli = createCac__default();
const defaultCwd = process.cwd();
function resolvePath(p, cwd = defaultCwd) {
  return path__default.isAbsolute(p) ? p : path__default.resolve(cwd, p);
}
cli.command("build [...files]", "Build files").option("--out <dir>", "Output directory").option("--cwd <cwd>", "Current working directory").action(async (files, options) => {
  const { cwd, out } = options;
  for (const file of files) {
    const entry = resolvePath(file, resolvePath(cwd));
    if (!fs__default.existsSync(entry)) {
      console.log(`${file} isn't existed! skipped`);
      continue;
    }
    const ctx = index.createContext();
    await ctx.process(entry);
    const code = ctx.generate();
    const filename = path__default.basename(entry, path__default.extname(entry));
    const outDir = out ? resolvePath(out, cwd) : path__default.dirname(entry);
    if (!fs__default.existsSync(outDir)) {
      fs__default.mkdirSync(outDir);
    }
    const target = path__default.resolve(outDir, filename + ".js");
    fs__default.writeFileSync(target, code, "utf8");
    console.log(`build successfully! file: ${target}`);
  }
});
cli.help();
cli.version("0.0.0");
cli.parse();
