import fs from 'node:fs/promises';
import fs$1 from 'node:fs';
import path from 'node:path';
import postcss from 'postcss';
import atImport from 'postcss-import';
import * as t from '@babel/types';
import babelGenerate from '@babel/generator';
import { pathToFileURL } from 'node:url';



// -- Unbuild CommonJS Shims --
import __cjs_url__ from 'url';
import __cjs_path__ from 'path';
import __cjs_mod__ from 'module';
const __filename = __cjs_url__.fileURLToPath(import.meta.url);
const __dirname = __cjs_path__.dirname(__filename);
const require = __cjs_mod__.createRequire(import.meta.url);
const layerNodes = {
  base: null,
  components: null,
  utilities: null
  // variants: null
};
const markedLayerKey = "__tw_layer_name__";
const layerNodesKeys = Object.keys(layerNodes);

const markLayerPlugin = () => {
  return {
    postcssPlugin: "postcss-css-to-tailwindcss-plugin-mark-layer-plugin",
    Once(root) {
      root.walkAtRules("layer", (rule) => {
        if (layerNodesKeys.includes(rule.params)) {
          const layerName = rule.params;
          for (const node of rule.nodes) {
            Object.defineProperty(node, markedLayerKey, {
              value: layerName,
              enumerable: false,
              configurable: true
            });
          }
          rule.parent?.insertBefore(rule, rule.nodes);
          rule.remove();
        }
      });
    }
  };
};
markLayerPlugin.postcss = true;
const extractLayerPlugin = (options) => {
  const { ctx } = options;
  return {
    postcssPlugin: "postcss-css-to-tailwindcss-plugin-extract-layer-plugin",
    OnceExit(root) {
      root.walkRules((rule) => {
        const layerName = rule[markedLayerKey];
        if (layerName && layerNodesKeys.includes(layerName)) {
          ctx.append(layerName, rule);
        }
      });
    }
  };
};
extractLayerPlugin.postcss = true;

function tryLoadSass() {
  return import('sass');
}
function tryLoadSassSync() {
  return require("sass");
}
function isExtSassFile(extname) {
  return extname === ".scss" || extname === ".sass";
}
async function sassCompile(path, sassOptions) {
  const { compile } = await tryLoadSass();
  const { css } = compile(path, sassOptions);
  return css;
}
function sassCompileSync(path, sassOptions) {
  const { compile } = tryLoadSassSync();
  const { css } = compile(path, sassOptions);
  return css;
}

const expandAST = ["addBase", "addComponents", "addUtilities", "theme", "addVariant", "config", "corePlugins", "e", "matchComponents", "matchUtilities", "matchVariant"].map(
  (x) => {
    return t.objectProperty(t.identifier(x), t.identifier(x));
  }
);
function expandAPI() {
  return expandAST;
}
function unwrapThemeFunctionArg(str) {
  if (str) {
    const start = str[0];
    const last = str.at(-1);
    if (start === '"' && last === '"' || start === "'" && last === "'") {
      return str.slice(1, -1);
    }
  }
  return str;
}
function makeObjectProperty(key, value) {
  return t.objectProperty(t.stringLiteral(key), value);
}
function makeObjectExpression(nodes) {
  return nodes.reduce((acc, _cur) => {
    if (_cur.type === "rule") {
      const cur = _cur;
      acc.push(
        t.objectProperty(
          t.stringLiteral(cur.selector),
          t.objectExpression(
            cur.nodes.reduce((cacc, ccur) => {
              if (ccur.type === "decl") {
                const v = ccur.value;
                const arr = [...v.matchAll(/theme\(([^()]+?)\)/g)];
                if (arr.length > 0) {
                  const first = arr[0];
                  if (first[0] === v) {
                    cacc.push(makeObjectProperty(ccur.prop, t.callExpression(t.identifier("theme"), [t.stringLiteral(unwrapThemeFunctionArg(first[1]))])));
                  } else {
                    let p = 0;
                    const quasis = [];
                    const expressions = [];
                    for (let i = 0; i < arr.length; i++) {
                      const hit = arr[i];
                      if (typeof hit.index === "number") {
                        const raw = v.slice(p, hit.index);
                        p = hit.index + hit[0].length;
                        quasis.push(t.templateElement({ raw }));
                        expressions.push(t.callExpression(t.identifier("theme"), [t.stringLiteral(unwrapThemeFunctionArg(hit[1]))]));
                        if (i === arr.length - 1) {
                          quasis.push(t.templateElement({ raw: v.slice(p) }));
                        }
                      }
                    }
                    cacc.push(makeObjectProperty(ccur.prop, t.templateLiteral(quasis, expressions)));
                  }
                } else {
                  cacc.push(makeObjectProperty(ccur.prop, t.stringLiteral(v)));
                }
              }
              return cacc;
            }, [])
          )
        )
      );
    }
    return acc;
  }, []);
}
function createGenerator() {
  const callFnId = "_plugin";
  const pluginName = "css2TwPlugin";
  const requireStatement = t.variableDeclaration("const", [
    t.variableDeclarator(t.identifier(callFnId), t.callExpression(t.identifier("require"), [t.stringLiteral("tailwindcss/plugin")]))
  ]);
  const exportsStatement = t.expressionStatement(t.assignmentExpression("=", t.memberExpression(t.identifier("module"), t.identifier("exports")), t.identifier(pluginName)));
  function generate(ctx) {
    const ast = t.file(
      t.program([
        requireStatement,
        t.variableDeclaration("const", [
          t.variableDeclarator(
            t.identifier(pluginName),
            t.callExpression(t.identifier(callFnId), [
              t.functionExpression(
                null,
                [t.objectPattern(expandAPI())],
                t.blockStatement([
                  // addBase
                  t.expressionStatement(t.callExpression(t.identifier("addBase"), [t.objectExpression(makeObjectExpression(ctx.getNodes("base")))])),
                  // addComponents
                  t.expressionStatement(t.callExpression(t.identifier("addComponents"), [t.objectExpression(makeObjectExpression(ctx.getNodes("components")))])),
                  // addUtilities
                  t.expressionStatement(t.callExpression(t.identifier("addUtilities"), [t.objectExpression(makeObjectExpression(ctx.getNodes("utilities")))]))
                ])
              )
            ])
          )
        ]),
        exportsStatement
      ])
    );
    const res = babelGenerate(ast);
    return res.code;
  }
  return {
    generate
  };
}

function isObject(value) {
  return value !== null && typeof value === "object";
}
function _defu(baseObject, defaults, namespace = ".", merger) {
  if (!isObject(defaults)) {
    return _defu(baseObject, {}, namespace, merger);
  }
  const object = Object.assign({}, defaults);
  for (const key in baseObject) {
    if (key === "__proto__" || key === "constructor") {
      continue;
    }
    const value = baseObject[key];
    if (value === null || value === void 0) {
      continue;
    }
    if (merger && merger(object, key, value, namespace)) {
      continue;
    }
    if (Array.isArray(value) && Array.isArray(object[key])) {
      object[key] = [...value, ...object[key]];
    } else if (isObject(value) && isObject(object[key])) {
      object[key] = _defu(
        value,
        object[key],
        (namespace ? `${namespace}.` : "") + key.toString(),
        merger
      );
    } else {
      object[key] = value;
    }
  }
  return object;
}
function createDefu(merger) {
  return (...arguments_) => (
    // eslint-disable-next-line unicorn/no-array-reduce
    arguments_.reduce((p, c) => _defu(p, c, "", merger), {})
  );
}
const defu = createDefu();

function getDefaults() {
  const pkgPath = require.resolve("tailwindcss/package.json");
  const nodeModulesPath = path.dirname(path.dirname(pkgPath));
  return {
    atImportOptions: {},
    sassOptions: {
      importers: [
        {
          // https://github.com/webpack-contrib/sass-loader/blob/master/src/index.js#L55
          findFileUrl(url) {
            const twUrl = pathToFileURL(path.resolve(nodeModulesPath, url));
            return new URL(twUrl);
          }
        }
      ]
    },
    tailwindcssConfig: ""
  };
}
function getOptions(opts) {
  return defu(opts, getDefaults());
}

function createContext(opts) {
  const options = getOptions(opts);
  const layersMap = /* @__PURE__ */ new Map();
  function resetLayersMap() {
    layersMap.set("base", []);
    layersMap.set("components", []);
    layersMap.set("utilities", []);
  }
  resetLayersMap();
  function getNodes(key) {
    const arrRef = layersMap.get(key);
    if (Array.isArray(arrRef)) {
      return arrRef;
    }
    const arr = [];
    layersMap.set(key, arr);
    return arr;
  }
  function append(key, nodes) {
    const arr = getNodes(key);
    if (Array.isArray(nodes)) {
      arr.push(...nodes);
    } else {
      arr.push(nodes);
    }
  }
  const generator = createGenerator();
  return {
    layersMap,
    append,
    getNodes,
    getPluginsSync() {
      const { tailwindcssConfig } = options;
      const opt = { ctx: this };
      const plugins = [markLayerPlugin()];
      if (tailwindcssConfig) {
        const tailwindcss = require("tailwindcss");
        plugins.push(tailwindcss(tailwindcssConfig));
      }
      plugins.push(extractLayerPlugin(opt));
      return plugins;
    },
    async getPlugins() {
      const { tailwindcssConfig } = options;
      const opt = { ctx: this };
      const plugins = [markLayerPlugin()];
      if (tailwindcssConfig) {
        const { default: tailwindcss } = await import('tailwindcss');
        plugins.push(tailwindcss(tailwindcssConfig));
      }
      plugins.push(extractLayerPlugin(opt));
      return plugins;
    },
    async process(entry) {
      const { sassOptions, atImportOptions } = options;
      const plugins = await this.getPlugins();
      let css;
      const extname = path.extname(entry);
      if (isExtSassFile(extname)) {
        try {
          css = await sassCompile(entry, sassOptions);
        } catch {
          throw new Error(`file: ${path} is skipped. please confirm you have installed \`sass\`!`);
        }
      } else {
        css = await fs.readFile(entry, "utf8");
        atImportOptions.root = path.dirname(entry);
        plugins.unshift(atImport(atImportOptions));
      }
      const res = await postcss(plugins).process(css, {
        from: void 0
      });
      return res;
    },
    processSync(entry) {
      const { sassOptions, atImportOptions } = options;
      const plugins = this.getPluginsSync();
      let css;
      const extname = path.extname(entry);
      if (isExtSassFile(extname)) {
        try {
          css = sassCompileSync(entry, sassOptions);
        } catch {
          throw new Error(`file: ${path} is skipped. please confirm you have installed \`sass\`!`);
        }
      } else {
        css = fs$1.readFileSync(entry, "utf8");
        atImportOptions.root = path.dirname(entry);
      }
      const res = postcss(plugins).process(css).sync();
      return res;
    },
    generate: function() {
      return generator.generate(this);
    }
  };
}

export { createContext };
